function widget_salesman_ranking_with_objective(widget) {
  "use strict";

  var Qweb = openerp.web.qweb;

  widget.SalesmanRankingObjective = widget.Base.extend({
    template: 'SalesmanRankingObjective',
    init: function (parent) {
      this._super(parent, {
        width: 12,
        height: 4
      });
    },

    start: function () {
      var self = this;
      self.fetchInitial();
    },

    fetchInitial: function(){
      var self = this;
      self.fetchDataSQL().then(function (DataSQL) {
        return DataSQL;
      }).then(function(DataSQL) {
        self.ResCompany = DataSQL.company;
        self.AccountInvoice = DataSQL.invoices;
        self.PosOrder = DataSQL.orders;
        self.ResUser = DataSQL.users;
        self.Objective = DataSQL.objective;
        return self.SalesmanRanking();
      });
    },

    fetchDataSQL: function() {
      var data = $.get('/dashboard-RankingSalesmanObjective');
      return data;
    },

    getPosOrder:function(id) {
      var self = this;
      return _.flatten(_.filter(self.PosOrder,function (inv) {
        return inv.user_id === id;
      }));
    },

    getAccountInvoice:function(id) {
      var self = this;
      return _.flatten(_.filter(self.AccountInvoice,function (inv) {
        return inv.user_id === id && inv.type == 'out_invoice';
      }));
    },

    getAccountInvoiceRefund:function(id) {
      var self = this;
      return _.flatten(_.filter(self.AccountInvoice,function (inv) {
        return inv.user_id === id && inv.type == 'out_refund';
      }));
    },

    getUserObjective:function(id) {
      var self = this;
      return _.map(_.filter(self.Objective,function (inv) {
        return inv.user_id === id;
      }), function(map){
        return map.sale_objective;
      });
    },

    SalesmanRanking: function() {
      var self = this;
      var ranking = [];
      var ResUser = self.ResUser;

      _.each(ResUser, function (item) {
        var PosOrder = self.getPosOrder(item.id);
        var AccountInvoice = self.getAccountInvoice(item.id);
        var AccountInvoiceRefund = self.getAccountInvoiceRefund(item.id);
        var UserObjective = self.getUserObjective(item.id);

        var pos_order_amount = _.reduce(_.map(PosOrder,function(item) {
          return item.amount;
        }),function(memo, num) {
          return memo + num;
        },0);

        var account_invoice_amount = _.reduce(_.map(AccountInvoice,function(item) {
          return item.amount;
        }),function(memo, num) {
          return memo + num;
        },0);

        var account_invoice_refund_amount = _.reduce(_.map(AccountInvoiceRefund,function(item) {
          return item.amount;
        }),function(memo, num) {
          return memo + num;
        },0);

        var amount = (pos_order_amount + account_invoice_amount) - account_invoice_refund_amount;

        if(amount > 0 ){
          ranking.push({
            name: item.name,
            amount: amount,
            objective: UserObjective[0],
          });
        }
      });

      ranking.sort(function(a, b) {
        return b.amount - a.amount
      });

      self.BuildChart(ranking);
    },

    BuildChart: function (ranking){
      var self = this;
      var label = [];
      var bodySale = [];
      var bodyObjective = [];
      var item;
      var CurrencyBase = self.ResCompany[0].currency_id;

      for (var i = 0; i < ranking.length; i++) {
        item = ranking[i];
        label.push(item.name);
        bodySale.push(item.amount);
        bodyObjective.push(item.objective);
      }

      var name = '.ranking-salesman-objective-chart';
      var chart = new widget.DashboardChartWidget2(self);
      chart.BuildBarChart2(name,label,bodyObjective,bodySale,CurrencyBase);
    }
  });
}
